"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("@gitsync/config");
const log_1 = require("@gitsync/log");
const execa = require("execa");
const fs_1 = require("fs");
const chalk_theme_1 = require("chalk-theme");
let command = {
    handler: () => {
    }
};
command.command = 'exec [cmd] [args..]';
command.describe = 'Execute command in the relative repositories directory';
command.builder = {
    cmd: {
        describe: 'The command to execute',
        type: 'string',
    },
    args: {
        describe: 'The arguments pass to the command',
        default: [],
        type: 'array',
    },
    include: {
        describe: 'Include only source directory matching the given glob',
        default: [],
        type: 'array',
    },
    exclude: {
        describe: 'Exclude source directory matching the given glob',
        default: [],
        type: 'array',
    }
};
command.handler = async (argv) => {
    argv.args || (argv.args = []);
    argv.include || (argv.include = []);
    argv.exclude || (argv.exclude = []);
    const extraArgs = argv['--'] || [];
    const cmd = argv.cmd || extraArgs.shift();
    const args = argv.args.concat(extraArgs);
    if (!cmd) {
        log_1.default.error('Require command argument to execute');
        process.exitCode = 1;
        return;
    }
    const config = new config_1.Config();
    config.checkFileExist();
    const repos = config.filterReposBySourceDir(argv.include, argv.exclude);
    for (const repo of repos) {
        const repoDir = await config.getRepoDirByRepo(repo);
        if (!await isDir(repoDir)) {
            log_1.default.warn(`Repository directory "${chalk_theme_1.default.warning(repoDir)}" does not exist, skipping`);
            continue;
        }
        log_1.default.info(`Executing command in ${chalk_theme_1.default.info(repoDir)}`);
        try {
            const result = await execa(cmd, args, {
                cwd: repoDir
            });
            log_1.default.info(result.all);
        }
        catch (e) {
            process.exitCode = e.exitCode;
            log_1.default.warn(`${e.message} ${e.all}`);
        }
    }
    log_1.default.info('Done!');
};
async function isDir(dir) {
    try {
        return (await fs_1.promises.stat(dir)).isDirectory();
    }
    catch (e) {
        return false;
    }
}
exports.default = command;
